# -*- coding: iso_8859_1 -*-

from aspylib import astro, photometry, conf
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	Purpose 
#------------------------------------------------------------------------------------
#
#	Calculates the instrumental magnitudes of asteroids and variable stars over a series of images.
#	For more explanations see: http://www.aspylib.com/doc/using_photometry_script.html
#
#------------------------------------------------------------------------------------
#	Input parameters (basic)
#------------------------------------------------------------------------------------

#--- calibrated images ---
Folder = u"C:\\images\\"
Imagelist = [Folder + "images-calibrated-" + str(i+1) + ".fit" for i in range(100)]

#radii for aperture photometry (pixels)
sizes = [[2,8,40],[2.5,8,40],[3,8,40],[3.5,8,40],[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40]]
#sizes = [[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40],[6.5,10,45],[7,10,45],[7.5,12,45],[8,12,45],[8.5,14,50],[9,14,50]]
#sizes = [[8,16,45],[8.5,16,50],[9,16,50],[9.5,16,55],[10,18,50],[10.5,18,50],[11,18,50],[11.5,18,50],[12,20,50]]

#--- names of output files ---
generic_name = Folder + 'test_'

#--- flag to shortcut image shift calculations ---
flag_calc = True	#if True, checks images and calculates shifts. If False, read results from file

#--- format of CdR-CdL output file ---
format = 'DVv'		#can be 'DVR' or 'DVv'
					#If 'DVR' the instrument magnitude of target and ref stars are written to the file
					#If 'DVv' the final target magnitude is written to the file, together with the magnitude noise values
					
#------------------------------------------------------------------------------------
#	Input parameters (advanced)
#------------------------------------------------------------------------------------

#--- semi-size of square areas used for star max detection and star fitting ---
psf_fit_area = 8			#typically a few times (2-3) the star FWHM.
max_detection_area = 2		#max_detection_area = 2 is recommended. Can be SLIGHTLY increased for very large stars (FWHM >5 pixels).

#--- photometry method ---
#method = 'select'
#method = 'maximum'
method = 'centroid'

#--- names of output files ---
outputname1 = generic_name + "01_check_images.txt"
outputname2 = generic_name + "02_image_shifts.txt"
outputname3 = generic_name + "03_selected_stars.pdf"
outputname4 = generic_name + "04_inst_magnitudes.txt"
outputname5 = generic_name + "05_best_diff_magnitudes.txt"
outputname6 = generic_name + "06_CDRCDLreport.txt"

#####################################################################################
#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------
print
print '*********************************************************************'
print '      SCRIPT FOR APERTURE PHOTOMETRY'
print '*********************************************************************'

#------------------------------------------------------------------------------------
#	selects images
#------------------------------------------------------------------------------------

if flag_calc:
	#------ checking quality of measured frames ------
	print
	print '---------------------------------------------------------------------'
	print '      Step 1 : checking the input images'
	print '---------------------------------------------------------------------'
	print
	Imagelist2 = photometry.check_pictures_quality(Imagelist, outputname1)

	#------ calculating shifts ------
	print
	print '---------------------------------------------------------------------'
	print '      Step 2 : calculation of image shifts'
	print '---------------------------------------------------------------------'
	print
	Imagelist3, shifts = photometry.calculate_shifts(Imagelist2, outputname2, detection_area = psf_fit_area, nb_stars = 12)
else:
	Imagelist3, shifts = photometry.load_shifts(outputname2)
	plt.ion()
	plt.plot(shifts[:,0], 'b')
	plt.plot(shifts[:,1], 'g')
	plt.grid()
	plt.title('Image shifts: Vertical (blue), Horizontal (green)')
	plt.xlabel('Image number')
	plt.ylabel('Image shifts (pixels)')
	astro.maximize_window(conf.cmd)
	sx, sy = astro.get_window_size(conf.cmd)
	plt.draw()	
	fx, fy = astro.get_window_position("Figure 1")
	astro.move_window("Figure 1", [sx, fy])
	print
	raw_input("Press Enter to continue")
	plt.close()
	
#------------------------------------------------------------------------------------
#	calculates instrumental magnitudes
#------------------------------------------------------------------------------------

#------ date information ------
dateobs = astro.julian_day(astro.get_headervalues(Imagelist3, 'DATE-OBS'))

#------ select stars ------
print
print '---------------------------------------------------------------------'
print '      Step 3 : selecting targets for photometry'
print '---------------------------------------------------------------------'
print
if method=='select':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = 0, method = method)
elif method=='maximum':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = max_detection_area, method = method)
elif method=='centroid':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, detection_area = psf_fit_area, method = method)

#------ perform aperture photometry ------
print
print '---------------------------------------------------------------------'
print '      Step 4 : calculating aperture photometry'
print '---------------------------------------------------------------------'
print
mag = photometry.make_photometry(Imagelist3, sizes, xy, method=method)
photometry.write_magnitudes(Imagelist3, outputname4, names, xy, dateobs, sizes, mag)	

#now magnitudes can be accessed with mag[image, object, disk]

nb_obj = len([i for i in names if (i[0]=='m')or(i[0]=='v')])
nb_ref = len([i for i in names if (i[0]=='r')])

#------------------------------------------------------------------------------------
#	checking reference stars
#------------------------------------------------------------------------------------

print
print '---------------------------------------------------------------------'
print '      Step 5 : checking reference stars'
print '---------------------------------------------------------------------'
print
					
if nb_ref==1:
	print
	print "Only one reference stars selected.\nScript is stopped without calculating differential magnitudes..."

else:			
	#mag2 is a copy of mag with all the rejected values masked (numpy masked array)
	mag2, rejstar, rejdata = photometry.check_refstars(names, mag)		

	if (nb_ref-len(rejstar)<=1):	#the ref star check cannot be consistent
		print
		print "Not enough remaining reference stars.\nScript is stopped without calculating differential magnitudes..."

#------------------------------------------------------------------------------------
#	calculating differential magnitudes
#------------------------------------------------------------------------------------

	else:
		nb_best = 10
		
		print
		print '---------------------------------------------------------------------'
		print '      Step 6 : calculating optimal differential magnitudes'
		print '---------------------------------------------------------------------'
		print

		for i in range(nb_obj):
		
			mag_i, rejdata_i, rejdisk_i = photometry.filter_magnitudes(i, names, sizes, mag2, rejdata)
			ref_1, mag_1, sig_1 = photometry.best_magnitude(i, names, sizes, mag, rejstar, rejdata_i, rejdisk_i, nb=nb_best)
			photometry.write_best_magnitude(outputname5, Imagelist3, names, i, sizes, ref_1, mag_1, sig_1, dateobs, nb=nb_best, rejdata = rejdata_i, rejsizes = rejdisk_i, rejstar = rejstar)
			photometry.write_cdrcdl_report(outputname6, Imagelist3, names, i, mag, ref_1, mag_1, sig_1, dateobs, rejdata = rejdata_i, format=format)
			
			dateobs_rej = [dateobs[j] for j in range(len(Imagelist3)) if j not in rejdata_i]
			plt.ion()
			plt.figure()
			plt.plot(dateobs_rej, mag_1[0])
			if (np.max(mag_1[0])-np.min(mag_1[0]))<0.2:
				plt.ylim([np.mean(mag_1[0])-0.2,np.mean(mag_1[0])+0.2])
			plt.title('Best lightcurve for '+names[i])
			plt.grid()
			ax = plt.gca()
			ax.set_ylim(ax.get_ylim()[::-1])
			plt.draw()

			astro.maximize_window(conf.cmd)
			sx, sy = astro.get_window_size(conf.cmd)
			fx, fy = astro.get_window_position("Figure 1")
			astro.move_window("Figure 1", [sx, fy])

			if i<nb_obj-1:
				print
				raw_input("press Enter to continue")
				plt.close()
			else:
				print
				raw_input("finished, press Enter to close window")
