# -*- coding: iso_8859_1 -*-

from aspylib import astro, photometry, astrometry
import numpy as np
import warnings
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	Purpose = match an image with USNO-B1 and get information for specific stars
#------------------------------------------------------------------------------------
#
#	Given an image, this script:
#	- makes an astrometric reduction, 
#	- downloads a USNO-B1 catalog from Vizier
#	- plots the image and the positions of the USNO-B1 stars on it
#	For each USNO-B1 star, information (name, position, magnitudes) is displayed in the console if the user makes a right-click on the star
#
#------------------------------------------------------------------------------------
#	Input parameters
#------------------------------------------------------------------------------------

#--- input image ---
Folder = u"C:\\images\\"
Image = Folder + "image-1.fit"

#------ Vizier parameters ------
catalog = 'USNO-B1'
alpha0 = astro.hours_to_decdegrees('00:10:30.0')
delta0 = astro.degrees_to_decdegrees('+44:20:45.0')
position = [alpha0, delta0]		#[alpha, delta] in degrees
conesize = [60., 60.]		#[boxsize_alpha, boxsize_delta] in arcmin
constraint = 'B1mag<=16'

#------ options ------
text_display = True
plotting = True

#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------
#****** (1) loading image and star catalog **************************************************************

#--- loads image ---
image0 = astro.get_imagedata([Image])

#--- loads catalog ---
astrometry.send_query_vizier(catalog, position, conesize, constraint)
cat = astrometry.read_votable("vizier.xml", ordering = [True,'R1mag'], remove_duplicates = [True, ['RAJ2000', 'DEJ2000']])
proj = astrometry.project_stars(cat.RAJ2000, cat.DEJ2000, alpha0, delta0)
print "stars downloaded from Vizier:",len(cat)

#****** (2) astrometric solution *************************************************************************

coef = astrometry.find_astrometry_for_image(image0, proj, nbdetstars = [40,100], nbcatstars = [60,200], text_display = text_display, plotting=plotting)

#****** (3) matching detected stars **********************************************************************

back = astrometry.inverse_distorsion_law(coef[1],proj)

fig, ax = astro.display(image0)
ax.scatter(back[:,1],back[:,0],s=20,marker='+',color='g')
plt.sca(ax)
plt.xlim([0,np.shape(image0)[1]])
plt.ylim([0,np.shape(image0)[0]])
plt.grid()
plt.draw()

warnings.simplefilter("ignore") 
def on_click(event): 
	if event.inaxes is not None: 
		if event.button==3: 
			xy = [event.ydata,event.xdata]
			dist = np.sqrt((xy[1]-back[:,1])**2.+(xy[0]-back[:,0])**2.)
			indices = np.argsort(dist)
			print 
			print "------ object from USNO-B1.0 ------"
			print "right-clicked (vert,horiz) position: %(x).3f, %(y).3f" % {'x':xy[0],'y':xy[1]}
			print "USNO-B1.0 : ", cat['USNO-B1.0'][indices[0]]
			print "Alpha J2000 : ", astro.decdegrees_to_hours(cat.RAJ2000[indices[0]])
			print "Delta J2000 :", astro.decdegrees_to_degrees(cat.DEJ2000[indices[0]])
			print "B1 mag : %(B1).3f" % {'B1':cat.B1mag[indices[0]]}
			print "B2 mag : %(B2).3f" % {'B2':cat.B2mag[indices[0]]}
			print "R1 mag : %(R1).3f" % {'R1':cat.R1mag[indices[0]]}
			print "R2 mag : %(R2).3f" % {'R2':cat.R2mag[indices[0]]}
	else:
		fig.canvas.mpl_disconnect(cid) 
		fig.canvas.stop_event_loop_default() 		

cid = fig.canvas.mpl_connect('button_press_event', on_click) 
fig.canvas.start_event_loop_default() 
warnings.resetwarnings() 

raw_input("finished")
