# -*- coding: iso_8859_1 -*-

import numpy as np
import math

def air_mass(alpha, delta, lat, long, JJ):
	"""
	---------------------
	Purpose
	Calculates air mass ratio for specific location, time and sky coordinates of an object 
	---------------------
	Inputs
	* alpha (string) = right ascension, string with format '13:34:12.45'
	* delta (string) = declination, string with format '-06:34:23.1'
	* lat (string) = observing geodetic latitude, string with format '+52:09:12.34' 
	* long (string) = observing longitude, positive East of Greenwich, string with format '+04:12:23.3'
	* JJ (float) = julian date for which air mass ratio must be calculated
	Allowed input formats for alpha are:
		'13:34:12.45'
		'13:34:12'	
	Allowed input formats for delta, lat, long are:
		'-06:34:23.1'
		'-06:34:23'
		'+06:34:23.1'
		'+06:34:23'
		'06:34:23.1'
		'06:34:23'
	---------------------
	Output (float) = air mass ratio
	---------------------
	"""

	#alpha, delta, lat, long in degrees
	#long is positive East of Greenwich
	
	#sideral time at Greenwich, 0h UT [in hours]
	T = (JJ-2415020.)/36525.
	theta0 = 6.6460656 + 2400.051262*T + 0.00002581*T*T
	
	#sideral time at Greenwich [in degrees]
	theta = np.mod(theta0 + 1.002737908 * (JJ-np.floor(JJ) + 0.5) * 24., 24.) * 15.
	
	#angle horaire [in degrees]
	H = np.mod(theta + long - alpha, 360.)
	
	#elevation [in degrees]
	k = math.pi/180.
	elevation = (1./k)*np.arcsin(np.sin(lat*k)*np.sin(delta*k)+np.cos(lat*k)*np.cos(delta*k)*np.cos(H*k))
	
	#air mass ratio
	dist = 90.-elevation
	air_mass = (1./np.cos(dist*k))*(1.-0.0012*(np.tan(dist*k)**2.))
	
	return air_mass

def hours_to_decdegrees(hours):
	"""
	---------------------
	Purpose
	Transforms string 'hours:minutes:seconds' to decimal number in degrees
	---------------------
	Inputs
	* hours (string) = string to be converted
	No minus sign is allowed in the input string. Allowed formats are:
		'13:34:12.45'
		'13:34:12'
	---------------------
	Output (float) = result in degrees
	---------------------
	"""
	values = hours.split(":")
	return 15.*(np.float(values[0])+np.float(values[1])/60.+np.float(values[2])/3600.)
	
def degrees_to_decdegrees(degrees):
	"""
	---------------------
	Purpose
	Transforms string 'degrees:arcmin:arcsec' to decimal number in degrees
	---------------------
	Inputs
	* degrees (string) = string to be converted
	A plus or minus sign is allowed for the first character. Allowed formats are:
		'-06:34:23.1'
		'-06:34:23'
		'+06:34:23.1'
		'+06:34:23'
		'06:34:23.1'
		'06:34:23'
	---------------------
	Output (float) = result in degrees
	---------------------
	"""
	values = degrees.split(":")
	if values[0][0] == '-':
		result = -(np.abs(np.float(values[0]))+np.float(values[1])/60.+np.float(values[2])/3600.)
	else:
		result = np.float(values[0])+np.float(values[1])/60.+np.float(values[2])/3600.
	return result

def dates_to_juliandays(dates):
	"""
	---------------------
	Purpose
	Converts a list of values representing dates and provided in the format "YYYYMMDD.dddddd", into a list of corresponding Julian dates
	---------------------
	Inputs
	* dates (list) = list of floats, in the form [d1, d2, ... dn]. Each float number is a date, with a format equivalent to "20110321.981322" for 21.981322 March 2011.
	---------------------
	Output (list) = list of floats, in the form [j1, j2, ... jn]. Each float number "ji" is the Julian date that corresponds to "di".
	---------------------
	"""

	result = []
	for dd in dates:

		year = float(str(dd)[0:4])
		month = float(str(dd)[4:6])
		day = float(str(dd)[6:])
		
		if month<=2:
			month = month+12
			year = year-1
		
		a = np.trunc(year/100.)
		b = 2. - a + np.trunc(a/4.)
			
		result.append(np.trunc(365.25*year)+np.trunc(30.6001*(month+1.))+day+1720994.5+b)
	return result

def decdegrees_to_hours(angle):
	"""
	---------------------
	Purpose
	Transforms a decimal number in degrees to a string 'hours:minutes:seconds'
	---------------------
	Inputs
	* angles (float) = number to be converted
	---------------------
	Output (string) = result returned with format 'hours:minutes:seconds'
	---------------------
	"""
	value = np.mod(angle,360.)/15.
	hours = np.int(value)
	minutes = np.int((value-hours)*60.)
	seconds = (value-hours-minutes/60.)*3600.
	hrs = '%(hrs)02d' % {'hrs': hours}
	min = '%(min)02d' % {'min': minutes}
	sec = '%(sec)05.2f' % {'sec': seconds}

	return hrs+':'+min+':'+sec

def decdegrees_to_degrees(angle):
	"""
	---------------------
	Purpose
	Transforms a decimal number in degrees to a string 'degrees:arcminutes:arcseconds'
	---------------------
	Inputs
	* angles (float) = number to be converted
	---------------------
	Output (string) = result returned with format 'degrees:arcminutes:arcseconds'
	---------------------
	"""
	sign = np.sign(angle)
	degrees = np.int(np.abs(angle))
	arcminutes = np.int((np.abs(angle)-degrees)*60.)
	arcseconds = (np.abs(angle)-degrees-arcminutes/60.)*3600.
	if sign>=0:
		header = '+'
	else:
		header = '-'
	deg = '%(deg)02d' % {'deg': degrees}
	arcmin = '%(arcmin)02d' % {'arcmin': arcminutes}
	arcsec = '%(arcsec)04.1f' % {'arcsec': arcseconds}

	return header+deg+':'+arcmin+':'+arcsec
