# -*- coding: iso_8859_1 -*-

import numpy as np
import process


def fit_stars(image, stars, detection_area, maxshift = [False,5], error_messages = 1, full_info=False):
	"""
	---------------------
	Purpose
	This function calculates the accurate positions of some stars on one image, starting from their approximate positions.
	The calculation is done by fitting an elliptical gaussian PSF on each star.
	It can be controlled that the PSF centroid that is found is not too far from the initial position (parameter maxshift).
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* stars (2D Numpy array) = array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the approximate positions of the stars.
	* detection_area (integer, must be >0) = defines the +/- size of the square window used for star fitting (the square window has a size 2*detection_area+1).
	The window size must be large enough to include the stars for the PSF fit, but small enough not to include contributions from other stars.
	* maxshift (list) = optional argument. 2 elements list with the form [boolean, integer]. The accurate positions are rejected if their distance to the initial approximate position is larger than a given limit.
		- if (maxshift[0]==True) the limit is maxshift[1]
		- if (maxshift[0]==False) the limit is detection_area
	If the shift is too large then the initial star position is returned.
	* error_messages (integer) = optional argument. Defines the warning messages which are displayed.
	If error_messages = 0 no warnings are displayed.
	If error_messages = 1 short warnings are displayed, giving the number of occurrence for each problem.
	If error_messages = 2 detailed warnings are displayed, listing all stars for which a particular problem occurred.
	* full_info (boolean) = optional argument. Depending on its value, the function will return different results (see explanations below).
	---------------------
	Possible warnings
	- star can be too close from image border for psf fit
	- accurate position can be too much distant from initial position (= poor psf fit)
	---------------------
	Output (2D Numpy array)
	If full_info = False (default), then the output is a Numpy array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains respectively, for each star:
	- the star centroid x and y positions (fit result)
	If full_info = True, then the output is a Numpy array with the form [[max1,floor1,height1,x1,y1,fhwmS1,fhwmL1,angle1],...,[maxn,floorn,heightn,xn,yn,fhwmSn,fhwmLn,anglen]], that contains respectively, for each star:
	- the value of the star maximum signal,
	- the sky background (fit result),
	- the PSF height (fit result),
	- the star centroid x and y positions (fit results), 
	- the smallest and largest full width half maximum (fit result) 
	- the angle, measured clockwise starting from the vertical direction, of the direction with the largest fwhm (fit result), and expressed in degrees. The direction of the smallest fwhm is obtained by adding 90 deg to this angle.
	---------------------
	"""
	#things to be controlled:
	#1- input position can be np.nan
	#2- input position can be too close to border to make psf fit
	#3- output position can be too far away from input position, with shift >= maxshift
	#4- output position can be too far away from input position, with shift >= detection_area and then the risk of being outside the image
	#output position is not controlled if (full_info==True)
	
	#image sizes
	size_x = image.shape[0] - detection_area - 1
	size_y = image.shape[1] - detection_area - 1
				
	#finding accurate star centers
	stars2 = []
	error = []

	if full_info:
		for i,xy in enumerate(stars):
			if np.isnan(xy[0]*xy[1]):
				stars2.append([np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan])
			elif (xy[0]<detection_area)or(xy[0]>size_x)or(xy[1]<detection_area)or(xy[1]>size_y):
				stars2.append([np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan])
				error.append(i+1)
			else:
				p = process.fit_gauss_elliptical([xy[0]-detection_area,xy[1]-detection_area],image[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1])
				max, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle = p
				stars2.append([max, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle])
		stars2 = np.array(stars2)

	else:
		for i,xy in enumerate(stars):
			if np.isnan(xy[0]*xy[1]):
				stars2.append([np.nan, np.nan])
			elif (xy[0]<detection_area)or(xy[0]>size_x)or(xy[1]<detection_area)or(xy[1]>size_y):
				stars2.append([np.nan, np.nan])
				error.append(i+1)
			else:
				p = process.fit_gauss_elliptical([xy[0]-detection_area,xy[1]-detection_area],image[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1])
				max, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle = p
				stars2.append([mean_x, mean_y])					
		stars2 = np.array(stars2)

		if maxshift[0]:
			max_shift = min(maxshift[1],detection_area)
		else:
			max_shift = detection_area

		test = (np.abs(stars2[:,0]-stars[:,0]) < max_shift)*(np.abs(stars2[:,1]-stars[:,1]) < max_shift) + (np.isnan(stars2[:,0]))	#the last term is needed to have correct error messages
		test2 = np.vstack([test,test]).transpose()
		stars2 = stars2*test2 + stars*(~test2)

	#displaying error messages
	if error_messages==1:
		if len(error)>0:
			print 'astro.fit_stars():',len(error),'/',len(stars),'object(s) too close from image border'
		if not(full_info):
			if np.sum(~test)>0:
				print 'astro.fit_stars():',np.sum(~test),'/',len(stars),'object(s) with poor psf fit'

	elif error_messages==2:
		if len(error)>0:
			print 'astro.fit_stars():',len(error),'/',len(stars),'object(s) too close from image border:'+str(error)[1:-1]
		if not(full_info):
			if np.sum(~test)>0:
				print 'astro.fit_stars():',np.sum(~test),'/',len(stars),'object(s) with poor psf fit:'+str(list(np.arange(1,len(~test)+1)[~test]))[1:-1]

	return stars2

def centroid_stars(image, stars, detection_area, saturation = [False,0], maxshift = 2, error_messages = 1):
	"""
	---------------------
	Purpose
	This function calculates the accurate positions of some stars on one image, starting from their approximate positions.
	The star centroid is calculated with an approximate formula proposed by N.Kaiser (A photometric study of the supercluster MS0302 with the UH8K CCD camera - image processing and object catalogs, arXiv:astro-ph/9907229v1 16 Jul 1999).
	The centroid calculation involves finding the star local maximum. It can be checked that the star local maximum is not above saturation (parameter saturation).
	It can be also checked that the star centroid that is found is not too far from the star local maximum (parameter maxshift).
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* stars (2D Numpy array) = array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the approximate positions of the stars.
	* detection_area (integer, must be >0) = defines the +/- size of the square window used to find the star maximum (the square window has a size 2*detection_area+1).
	The window size must be large enough to find the stars maxima, but small enough not to include contributions from other stars.
	* saturation (list) = optional argument. Two elements lists: if saturation[0] is True, a warning is displayed if the star maximum signal is >= than saturation[1].
	* maxshift (integer) = optional argument. The centroid positions are rejected if their distance to the star maximum position is larger than maxshift. Then the position of the star maximum is returned.
	* error_messages (integer) = optional argument. Defines the warning messages which are displayed.
	If error_messages = 0 no warnings are displayed.
	If error_messages = 1 short warnings are displayed, giving the number of occurrence for each problem.
	If error_messages = 2 detailed warnings are displayed, listing all stars for which a particular problem occurred.
	---------------------
	Possible warnings
	- star can be too close from image border to find maximum or calculate centroid
	- star maximum signal can be above saturation
	- the centroid position can be too much distant from initial position (= poor centroid position)
	---------------------
	Output (2D Numpy array) = Numpy array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the centroid x and y positions for each star.
	---------------------
	"""

	#--- other possibility ---
	#stars2 = np.array(map(lambda xy: np.array(np.where(image[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1]== \
	#np.max(np.max(image[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1])))).transpose()[0] \
	#+ xy - detection_area, stars))
	#-------------------------

	#things to be controlled:
	#1- input position can be too close to border to make max detection
	#2- star can be saturated
	#3- calculated maximum X and Y positions +/-1 must be within image boundaries to allow to calculate derivatives
	#4- final centroid position must be within image boundaries

	#image sizes
	size_0 = detection_area + max(1,maxshift)
	size_x = image.shape[0] - detection_area - 1 - max(1,maxshift)
	size_y = image.shape[1] - detection_area - 1 - max(1,maxshift)

	#finding centroids
	stars2 = []
	error_position = []
	error_saturate = []
	for i,xy in enumerate(stars):
		if (xy[0]<size_0)or(xy[0]>size_x)or(xy[1]<size_0)or(xy[1]>size_y):
			stars2.append([np.nan, np.nan])
			error_position.append(i+1)
		else:
			subimage = image[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1]
			x,y = divmod(subimage.argmax(),subimage.shape[1])
			if saturation[0]:
				if subimage[x,y]>=saturation[1]:
					stars2.append([np.nan, np.nan])
					error_saturate.append(i+1)
				else:
					stars2.append([x + xy[0] - detection_area, y + xy[1] - detection_area])
			else:
				stars2.append([x + xy[0] - detection_area, y + xy[1] - detection_area])

	stars2 = np.array(stars2)
	ind = np.array(np.nan_to_num(stars2),dtype=np.int)

	maxi = image[ind[:,0],ind[:,1]]
	Fx = (image[ind[:,0]+1,ind[:,1]] - image[ind[:,0]-1,ind[:,1]])/2.
	Fy = (image[ind[:,0],ind[:,1]+1] - image[ind[:,0],ind[:,1]-1])/2.
	Fxx = image[ind[:,0]+1,ind[:,1]] - 2.*maxi + image[ind[:,0]-1,ind[:,1]] + 1e-20
	Fyy = image[ind[:,0],ind[:,1]+1] - 2.*maxi + image[ind[:,0],ind[:,1]-1] + 1e-20
	
	#checks for shift smaller than maxshift on both directions
	Sx = Fx/Fxx
	Sy = Fy/Fyy
	test_ok = (np.abs(Sx)<maxshift)*(np.abs(Sy)<maxshift) + (np.isnan(stars2[:,0]))		#the second term is needed to have correct error messages

	#stars3 is automatically set to np.nan where stars2 is equal to np.nan
	#so no need for this equation: stars3 = stars3 + 0.*stars2
	stars3 = np.vstack([stars2[:,0]-Sx*test_ok,stars2[:,1]-Sy*test_ok]).transpose()
	
	#warnings
	if error_messages == 1:
		if len(error_position)>0:
			print 'astro.centroid_stars():',len(error_position),'/',len(stars),'object(s) too close from image border'
		if len(error_saturate)>0:
			print 'astro.centroid_stars():',len(error_saturate),'/',len(stars),'object(s) above saturation'
		if np.sum(~test_ok)>0:
			print 'astro.centroid_stars():',np.sum(~test_ok),'/',len(stars),'object(s) with poor centroid calculation'

	elif error_messages == 2:
		if len(error_position)>0:
			print 'astro.centroid_stars():',len(error_position),'/',len(stars),'object(s) too close from image border:'+str(error_position)[1:-1]
		if len(error_saturate)>0:
			print 'astro.centroid_stars():',len(error_saturate),'/',len(stars),'object(s) above saturation:'+str(error_saturate)[1:-1]
		if np.sum(~test_ok)>0:
			print 'astro.centroid_stars():',np.sum(~test_ok),'/',len(stars),'object(s) with poor centroid calculation:'+str(list(np.arange(1,len(~test_ok)+1)[~test_ok]))[1:-1]
		
	return stars3

def max_stars(image, stars, detection_area, saturation = [False,0], error_messages = 1):
	"""
	---------------------
	Purpose
	This function calculates the positions of the star local maxima, starting from an estimate.
	It can be checked that the local maxima are not above saturation (parameter saturation).
	---------------------
	Inputs
	* image (2D Numpy array) = the data of one FITS image, as obtained for instance by reading one FITS image file with the function astro.get_imagedata().
	* stars (2D Numpy array) = array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the approximate positions of the stars.
	* detection_area (integer, must be >0) = defines the +/- size of the square window used to find the star maximum (the square window has a size 2*detection_area+1).
	The window size must be large enough to find the stars maxima, but small enough not to include contributions from other stars.
	* saturation (list) = optional argument. Two elements lists: if saturation[0] is True, a warning is displayed if the star maximum signal is >= than saturation[1].
	* error_messages (integer) = optional argument. Defines the warning messages which are displayed.
	If error_messages = 0 no warnings are displayed.
	If error_messages = 1 short warnings are displayed, giving the number of occurrence for each problem.
	If error_messages = 2 detailed warnings are displayed, listing all stars for which a particular problem occurred.
	---------------------
	Possible warnings
	- star can be too close from image border to find maximum
	- star maximum signal can be above saturation
	---------------------
	Output (2D Numpy array) = Numpy array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the local max x and y positions for each star.
	---------------------
	"""

	#things to be controlled:
	#1- input position can be too close to border to make max detection
	#2- star can be saturated

	#image sizes
	size_x = image.shape[0] - detection_area - 1
	size_y = image.shape[1] - detection_area - 1

	#finding local max
	stars2 = []
	error_position = []
	error_saturate = []
	for i,xy in enumerate(stars):
		if (xy[0]<detection_area)or(xy[0]>size_x)or(xy[1]<detection_area)or(xy[1]>size_y):
			stars2.append([np.nan, np.nan])
			error_position.append(i+1)
		else:
			subimage = image[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1]
			x,y = divmod(subimage.argmax(),subimage.shape[1])
			if saturation[0]:
				if subimage[x,y]>=saturation[1]:
					stars2.append([np.nan, np.nan])
					error_saturate.append(i+1)
				else:
					stars2.append([x + xy[0] - detection_area, y + xy[1] - detection_area])
			else:
				stars2.append([x + xy[0] - detection_area, y + xy[1] - detection_area])

	stars2 = np.array(stars2)
			
	#warnings
	if error_messages == 1:
		if len(error_position)>0:
			print 'astro.max_stars():',len(error_position),'/',len(stars),'object(s) too close from image border'
		if len(error_saturate)>0:
			print 'astro.max_stars():',len(error_saturate),'/',len(stars),'object(s) above saturation'
	elif error_messages == 2:
		if len(error_position)>0:
			print 'astro.max_stars():',len(error_position),'/',len(stars),'object(s) too close from image border:'+str(error_position)[1:-1]
		if len(error_saturate)>0:
			print 'astro.max_stars():',len(error_saturate),'/',len(stars),'object(s) above saturation:'+str(error_saturate)[1:-1]

	return stars2

def remove_duplicates(stars):
	"""
	---------------------
	Purpose
	This function removes the duplicates from a list of star positions in the form [[x1,y1],[x2,y2],...,[xn,yn]]
	Stars i and j are duplicates if (xi==xj) and (yi==yj).
	---------------------
	Inputs
	* stars (2D Numpy array) = array with the form [[x1,y1],[x2,y2],...,[xn,yn]], that contains the positions of the stars with possible duplicates.
	---------------------
	Outputs = stars2, indices
	* stars2 (2D Numpy array) = the array of star positions, after the duplicates have been removed
	* indices (list) = indices that have been used to remove the duplicates. We have stars2 = stars[indices].
	---------------------
	"""

	stars2 = np.core.records.fromarrays(list(stars.transpose()))
	stars2, indices = np.unique(stars2, return_index=True)
	indices = np.sort(indices)
	stars2 = stars[indices]
	return stars2, indices
