# -*- coding: iso_8859_1 -*-

import numpy as np
import sys

def project_stars(alpha, delta, alpha0, delta0):
	"""
	---------------------
	Purpose
	Transforming a list of (alpha, delta) star coordinates into (x, y) cartesian coordinates using an orthographic projection, which is conformal (=does not change the shapes).
	The transformation is described in:
	N.Kaiser, G.Wilson, G.Luppino, H.Dahle "A photometric study of the supercluster MS0302 with the UH8K CCD camera - image processing and object catalogs"
	arXiv:astro-ph/9907229v1 (16 Jul 1999)
	---------------------
	Inputs
	* alpha (Numpy array) = right ascension, in degrees, for the stars to be transformed.
	* delta (Numpy array) = declination, in degrees, for the stars to be transformed.
	* alpha0 (float) = right ascension, in degrees, for the center of the field of view.
	* delta0 (float) = declination, in degrees, for the center of the field of view.
	The point with coordinates (alpha0, delta0) is the point of the projection plane that is tangent to the unit sphere.
	---------------------
	Output (Numpy array) = Numpy array such that: Output[:,0] = vertical projected coordinate, Output[:,1] = horizontal projected coordinate.
	---------------------
	"""
		
	alpha0 = alpha0*np.pi/180.
	delta0 = delta0*np.pi/180.

	axis = [np.cos(delta0)*np.cos(alpha0), np.cos(delta0)*np.sin(alpha0), np.sin(delta0)]
	rvert = [-np.sin(delta0)*np.cos(alpha0), -np.sin(delta0)*np.sin(alpha0), np.cos(delta0)]
	rhoriz = [np.sin(alpha0), -np.cos(alpha0), 0.0]
	
	stars_X = np.cos(delta*np.pi/180.)*np.cos(alpha*np.pi/180.)
	stars_Y = np.cos(delta*np.pi/180.)*np.sin(alpha*np.pi/180.)
	stars_Z = np.sin(delta*np.pi/180.)
	
	dot_axis = stars_X*axis[0] + stars_Y*axis[1] + stars_Z*axis[2]
	dot_rvert = stars_X*rvert[0] + stars_Y*rvert[1] + stars_Z*rvert[2]
	dot_rhoriz = stars_X*rhoriz[0] + stars_Y*rhoriz[1] + stars_Z*rhoriz[2]
	
	return np.vstack([1.e6*(2.*dot_rvert) / (1.+dot_axis), 1.e6*(2.*dot_rhoriz) / (1.+dot_axis)]).transpose()

def inverse_project_stars(rv, rh, alpha0, delta0):
	"""
	---------------------
	Purpose
	Performs the inverse of the transformation made by the function astrometry.project_stars()
	---------------------
	Inputs
	* rv (Numpy array) = vertical projected coordinate, in degrees, for the stars to be transformed.
	* rh (Numpy array) =  horizontal projected coordinate, for the stars to be transformed.
	* alpha0 (float) = right ascension, in degrees, for the center of the field of view.
	* delta0 (float) = declination, in degrees, for the center of the field of view.
	The point with coordinates (alpha0, delta0) is the point of the projection plane that is tangent to the unit sphere.
	---------------------
	Output (Numpy array) = Numpy array such that: Output[:,0] = right ascension (in degrees), Output[:,1] = declination (in degrees).
	---------------------
	"""

	alpha0 = alpha0*np.pi/180.
	delta0 = delta0*np.pi/180.

	axis = [np.cos(delta0)*np.cos(alpha0), np.cos(delta0)*np.sin(alpha0), np.sin(delta0)]
	rvert = [-np.sin(delta0)*np.cos(alpha0), -np.sin(delta0)*np.sin(alpha0), np.cos(delta0)]
	rhoriz = [np.sin(alpha0), -np.cos(alpha0), 0.0]

	rv = rv*1.e-6
	rh = rh*1.e-6
	
	coef1 = 4.*rv / (4.+rv**2.+rh**2.)
	coef2 = 4.*rh / (4.+rv**2.+rh**2.)
	coef3 = (4.-rv**2.-rh**2.) / (4.+rv**2.+rh**2.)
	
	stars_X = coef1*rvert[0] + coef2*rhoriz[0] + coef3*axis[0]
	stars_Y = coef1*rvert[1] + coef2*rhoriz[1] + coef3*axis[1]
	stars_Z = coef1*rvert[2] + coef2*rhoriz[2] + coef3*axis[2]
	
	alpha = (180./np.pi)*np.arctan2(stars_Y, stars_X)
	delta = (180./np.pi)*np.arcsin(stars_Z)

	return np.vstack([alpha, delta]).transpose()
	
# def append_field(rec, name, arr, dtype=None):
    # arr = np.asarray(arr)
    # if dtype is None:
        # dtype = arr.dtype
    # newdtype = np.dtype(rec.dtype.descr + [(name, dtype)])
    # newrec = np.empty(rec.shape, dtype=newdtype)
    # for field in rec.dtype.fields:
        # newrec[field] = rec[field]
    # newrec[name] = arr
    # return newrec
