# -*- coding: iso_8859_1 -*-

from aspylib import astro
import astrometry2, astrometry3, astrometry5
import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize
import sys

def find_astrometry_for_image(image, proj, distcrit = 5, nbdetstars = [60,200], nbcatstars = [100,1000], text_display = True, plotting=True): 

	#purpose is to find astrometric coefficients, including distorsion, to transform image pixel coordinates into stereographic projected coordinates

	#*********************************************************************************************************
	#		(1) preparing inputs
	#*********************************************************************************************************

	#--- catalog subsets ---
	proj1 = proj[:nbcatstars[0],:]
	proj2 = proj[:nbcatstars[1],:]

	#--- finding stars on image ---
	stars = astro.find_stars(image, max(nbdetstars), saturation = [False,0], text_display = text_display)
	stars = astro.centroid_stars(image, stars, 2)
	stars, indices = astro.remove_duplicates(stars)
	stars1 = stars[:nbdetstars[0],:]
	stars2 = stars[:nbdetstars[1],:]

	#*********************************************************************************************************
	#		(2) first astrometric solution
	#*********************************************************************************************************

	#--- approximate solution ---
	#scaling + rotation + translation that transform the pixel coordinates into stereographic coordinates
	scaling, phi, dx, dy = astrometry2.approximate_astrometry(stars1, proj1, nb_iterations=2, resolution=1024)
	newcoef = [[scaling, phi, dx, dy],[]]		
	p0 = [dx, scaling*np.cos(phi), -scaling*np.sin(phi), 0., 0., 0., 0., 0., 0., 0., dy, scaling*np.sin(phi), scaling*np.cos(phi), 0., 0., 0., 0., 0., 0., 0.]
	
	#*********************************************************************************************************
	#		(3) checking + display results
	#*********************************************************************************************************
	
	#--- star transformation ---
	trans1 = astrometry5.transform_stars(newcoef[0], stars1)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans1,proj1,distcrit*newcoef[0][0])

	#--- checking ---
	if len(x)<10:
		print
		print "-----------------------------------------------------------------------"
		print "astrometric reduction failed, with only "+str(len(x))+" matching stars"
		print "-----------------------------------------------------------------------"
		sys.exit("Script execution stopped")
		
	if text_display:
		print
		print "--- searching approximate astrometry ---"
		print "nb of stars used (detected / from catalog):",len(stars1),"/",len(proj1)
		print "approximate astrometric coefficients:", scaling, phi, dx, dy
		print "nb of matched pairs:" ,len(x)
	
	if plotting:
		plt.figure()
		plt.scatter(trans1[:,1],trans1[:,0],s=20)
		plt.scatter(proj1[:,1],proj1[:,0],s=20,marker='+',color='r')
		plt.grid()
		plt.title("first star matching")
		plt.show()

	#*********************************************************************************************************
	#		(4) accurate astrometry
	#*********************************************************************************************************

	#--- star transformation ---
	trans2 = astrometry5.transform_stars(newcoef[0], stars2)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans2,proj2,distcrit*newcoef[0][0])

	#--- accurate astrometry ---
	if len(x)<20:
		print
		print "-----------------------------------------------------------------------"
		print "searching accurate astrometry:"
		print "not enough matching pairs for least square fit, an error will occur"
		print "-----------------------------------------------------------------------"
	coordinates = np.vstack([stars2[x,0], stars2[x,1], proj2[y,0], proj2[y,1]]).transpose()
	errfunc = lambda p, xy: (astrometry5.distorsion_law(p, xy[:,:2]) - xy[:,2:]).flatten() # Distance to the target function
	p1, success = optimize.leastsq(errfunc, p0, args=(coordinates))
	newcoef[1] = p1
		
	#*********************************************************************************************************
	#		(5) display results
	#*********************************************************************************************************

	if text_display:
		print
		print "--- searching accurate astrometry ---"
		print "nb of stars used (detected / from catalog):",len(stars2),"/",len(proj2)
		print "nb of matched pairs used for accurate astrometry:" ,len(x)
		print "astrometric coefficients:", p1

	# if plotting:
		# plt.figure()
		# plt.scatter(trans2[:,1],trans2[:,0],s=20)
		# plt.scatter(proj2[:,1],proj2[:,0],s=20,marker='+',color='r')
		# plt.grid()
		# plt.title("accurate star matching")
		# plt.show()

	return newcoef
